#pragma once
/**
 *  @file level1.h
 *  @date 12.12.2014
 *  @author mkoenig
 *  @brief 
 *
 *  $LastChangedDate$
 *  $LastChangedBy$
 *  $LastChangedRevision$
 *
 */

#include "cpp_odbc/level1/api.h"
#include "cpp_odbc/level1/unixodbc_backend.h"

namespace cpp_odbc {

/**
 * @brief This namespace contains classes which are VERY close to the unixODBC interface.
 *        The classes found here are consumed by functionality found in the cpp_odbc::level2
 *        namespace.
 */
namespace level1 {}


}
